﻿## Function to make labels...

source("f_makeqrcodes_2012-09-26-1357.R")

make.labels <- function (working.directory, input.file, type="pin", barcodes="TRUE", barcode.start=NA, prefix="") {
 setwd(working.directory)

 ## Printing out input parameters...
 print(paste("working directory:", working.directory))
 print(paste("input:", input.file))
 print(paste("label type:", type))

 ## Setting parameters determined by input...
 if (type=="pin")
  {
  width=18
  n.cols=10
  font.scale <- 0.29
  }
 if (type=="vial")
  {
  width=47
  n.cols=4
  font.scale <- 0.6
  }

 ## Loading data
 options(encoding="native.enc")
 data <- read.csv(input.file, colClasses="character")
 options(encoding="UTF-8")

 ## Preparing a data frame...
 colcode <- data$COLLECTION_CDE
 cat_num <- data$CAT_NUM
 country <- data$COUNTRY
 state <- data$STATE_PROV
 loc <- data$SPEC_LOCALITY
 lat <- data$DEC_LAT
 lon <- data$DEC_LONG
 unc <- data$COORDINATEUNCERTAINTYINMETERS
 unit <- rep("m", length(data$CAT_NUM))
 bdate <- data$BEGAN_DATE
 edate <- data$ENDED_DATE
 col <- data$COLLECTORS
 met <- data$COLLECTING_METHOD
 part <- data$PARTS
 sci.name <- data$SCIENTIFIC_NAME
 determiner <- data$IDENTIFIEDBY
 det.date <- data$MADE_DATE
 atts <- data$ATTRIBUTES

 data2 <- as.data.frame(cbind(colcode, cat_num, country, state, loc, lat, lon, unc, unit, bdate, edate, col, met, part, sci.name, determiner, det.date, atts),  stringsAsFactors=FALSE)

 ## cleaning...
 data2$sci.name <- gsub("#", "\\\\#{}", data2$sci.name)
 data2$loc <- gsub("&", "\\\\&{}", data2$loc)


 ## OK, now the work begins...
 ## First replacing "United States" with "USA"
 data2$country[data2$country == "United States"] <- "USA"

 ## Now for coordinates...
 data2$lat <- round(as.numeric(data2$lat), digits=5)
 data2$lon <- round(as.numeric(data2$lon), digits=5)
 data2$unc <- as.numeric(data2$unc)

 ## Changing m to km if this is better...
 data2$unc[is.na(data2$unc)] <- 0 
 data2$unit[data2$unc > 999] <- "km"
 data2$unc[data2$unc > 999] <- data2$unc[data2$unc > 999]/1000
 ## Maybe I will wait on this...

 ## Dates...
 ## Converting strings to dates...
 data2$bdate <- strptime(data2$bdate, format="%Y-%m-%d")
 data2$edate <- strptime(data2$edate, format="%Y-%m-%d")
 data2$det.date <- strptime(data2$det.date, format="%Y-%m-%d")
 ## Making an empty string for formatted dates.
 date.string <- rep(NA, dim(data2)[1])
 ## If the beginning and end dates are the same, this is easy...
 rows <- data2$bdate == data2$edate
 date.string[rows] <- format(data2$edate[rows], format="%d.%b.%Y")
 ## If the year and month are the same...
 rows <- format(data2$bdate, format="%b.%Y") == format(data2$edate, format="%b.%Y") & is.na(date.string)
 date.string[rows] <- paste(format(data2$bdate[rows], format="%d"), "-", format(data2$edate[rows], format="%d.%b.%Y"), sep="")
 ## If the months are different...
 rows <- format(data2$bdate, format="%Y") == format(data2$edate, format="%Y") & is.na(date.string)
 date.string[rows] <- paste(format(data2$bdate[rows], format="%d.%b"), "-", format(data2$edate[rows], format="%d.%b.%Y"), sep="")
 ## If the years differ...
 rows <- is.na(date.string)
 date.string[rows] <- paste(format(data2$bdate[rows], format="%d.\\-%b.\\-%Y"), "-", format(data2$edate[rows], format="%d.%b.%Y"),   sep="")
 data2 <- cbind(data2, date.string)
 ## Formatting det dates...
 data2$det.date <- format(data2$det.date, format="%d.%b.%Y")

 data2 <- data2[order(data2$cat_num),]
 datap <- data2#[data2$part == "whole organism (pinned)",]

 ## Determining the barcode series...
 if (!is.na(barcode.start))
  {
  l <- dim(data2)[1]
  barcode <- barcode.start:(barcode.start + (l-1))
  barcode <- paste(prefix, barcode, sep="")
  print(paste('barcode series: ', barcode[1], '-', barcode[l], sep=''))
  }

 ## Making QR codes if they are needed...
 if(type=="pin")
  {
  barcode.pdf <- make.qrcodes(barcode)
  }

 ## Getting ready to write the .tex file.
 options(encoding="UTF-8")
 file.name <- sub(".csv", ".tex", input.file)
 print(paste("output TeX file:", file.name))

 line <- paste('\\documentclass[10pt]{article}
\\usepackage{fontspec}
\\usepackage{multicol}
\\usepackage[margin=1cm]{geometry}
\\usepackage{rotating}
\\usepackage{wrapfig}

%% Font.
\\setmainfont[Scale=', font.scale, ', PunctuationSpace=0.5, WordSpace = 0.5]{DroidSansDotted90.ttf}
\\newfontfamily\\idfont[Scale=', font.scale, ', PunctuationSpace=0.5, WordSpace = 0.4, Weight=2]{DroidSansDotted-Bold90.ttf}
\\newfontfamily\\italicfont[Scale=', font.scale, ', PunctuationSpace=0.5, WordSpace = 0.4, Weight=2]{DroidSansDotted-Oblique80.ttf}
\\font\\barcode="[FRE3OF9X.TTF]":extend=1 at 23pt  % 
%% Inter-line spacing.
\\linespread{0.26}

%% Commands for typesetting labels.
\\renewcommand{\\fboxsep}{0.2mm}
\\newcommand{\\insectlabel}[1]{\\filbreak\\framebox[', width, 'mm][l]{\\parbox{', width-0.5, 'mm}{#1}}\\\\}
\\newcommand{\\viallabel}[1]{\\filbreak\\framebox[48mm][l]{\\parbox{', width, 'mm}{#1}}\\\\}
\\newcommand{\\idlabel}[1]{\\fbox{\\parbox{6mm}{\\centering\\idfont\\bfseries #1}}\\\\}
\\setlength{\\columnsep}{1mm}
\\setlength{\\parindent}{0mm}

\\begin{document}
\\begin{multicols}{', n.cols, '}', sep='')

 write(line, file.name, append=FALSE)

## Pinned labels...
if(type=="pin") {
 line <- "%% Begin pin labels..."
 write(line, file.name, append=TRUE)
 for (this.label in 1:dim(datap)[1])
  {
  ## Start label and add locality...
  line <- paste("\\filbreak\\insectlabel{
 \\begin{wrapfigure}{r}{6.3mm}
  \\vspace{-3.9mm}
  \\hspace{-0.4mm}
  \\begin{turn}{90}
   \\parbox{6mm}{\\hspace{0.1mm}", barcode[this.label], "\\\\
   \\hspace{-0.5mm}\\includegraphics[width=5.5mm]{", barcode.pdf[this.label], "}}
  \\end{turn}
  \\hspace{-5mm}
  \\vspace{-4.5mm}
 \\end{wrapfigure}  
 ", datap$country[this.label], ": ", datap$state[this.label], ", ", datap$loc[this.label],  sep="")
  ## Add coordinates...
  line <- paste(line, " ", datap$lat[this.label], "\\textdegree{}N ", -datap$lon[this.label], "\\textdegree{}W", sep="")
  if(datap$unc[this.label] > 0)
   {
   if(datap$unit[this.label] == "m")
    {
    line <- paste(line, " ±", round(datap$unc[this.label], digits=0), "m", sep="")
    } 
   if(datap$unit[this.label] == "km")
    {
    line <- paste(line, " ±", datap$unc[this.label], "km", sep="")
    } 
   }
  ## Add collectors...
  line <- paste(line, " ", datap$date.string[this.label], ". ", sep="")
  line <- paste(line, " ", datap$col[this.label], ".", sep="")
  ## Add method...
  if (!datap$met[this.label]=="")
   {
   line <- paste(line, " ", datap$met[this.label], ". ", sep="")
   }
  ## Add the id label...
  #line <- paste(line, "\\\\\\centering{\\barcode *K", datap$cat_num[this.label], "*}\\\\\\idfont{KNWR: Ento: ",    #datap$cat_num[this.label], "}", sep="")
  line <- paste(line, " KNWR: Ento: {\\idfont ", datap$cat_num[this.label], "}", sep="")
  ## Finish label...
  line <- paste(line, "}

", sep="")
  write(line, file.name, append=TRUE)
  }
 line <- "\\end{multicols}
 \\end{document}"
 write(line, file.name, append=TRUE)
}

## Vial labels...
if(type=="vial") {
 line <- "%% Begin pin labels..."
 write(line, file.name, append=TRUE)
 for (this.label in 1:dim(datap)[1])
  {
  ## Start label 
  line <- "\\filbreak\\viallabel{"
  line <- paste(line, "\\centering barcode: ", barcode[this.label], " \\hspace{4mm} {\\idfont KNWR:", datap$colcode[this.label], ":", datap$cat_num[this.label], "} ", sep="")
  line <- paste(line, "\\\\ \\centering {\\barcode *", barcode[this.label], "*}  ", sep="") 
  if (datap$colcode[this.label]=="Ento") {line <- paste(line, "\\hspace{0mm} {\\idfont 90-100\\% ETOH}", sep="")}
  line <- paste(line, "\\\\", sep="")

  ##and add locality...
  line <- paste(line, "\\vspace{-2pt}\\rule{", width, "mm}{0.3pt}\\\\\\raggedright ", datap$country[this.label], ":", datap$state[this.label], ", ", datap$loc[this.label],  sep="")
  ## Add coordinates...
  line <- paste(line, " ", datap$lat[this.label], "\\textdegree{}N ", -datap$lon[this.label], "\\textdegree{}W", sep="")
  if(datap$unc[this.label] > 0)
   {
   if(datap$unit[this.label] == "m")
    {
    line <- paste(line, " ±", round(datap$unc[this.label], digits=0), "m", sep="")
    } 
   if(datap$unit[this.label] == "km")
    {
    line <- paste(line, " ±", datap$unc[this.label], "km", sep="")
    } 
   }
  ## Add collectors...
  line <- paste(line, datap$date.string[this.label])
  line <- paste(line, " ", datap$col[this.label], ".", sep="")
  ## Add method...
  line <- paste(line, datap$met[this.label])
  ## det data...
  if(is.na(datap$det.date[this.label]))
   {
   line <- paste(line, "\\\\ \\vspace{-1pt}\\rule{", width, "mm}{0.3pt}\\\\ \\vspace{1pt}{", datap$sci.name[this.label], "} \\\\ det.\\ ", datap$determiner[this.label], sep="")
   }
  else
   {
  line <- paste(line, "\\\\ \\vspace{-1pt}\\rule{", width, "mm}{0.3pt}\\\\ \\vspace{1pt}{", datap$sci.name[this.label], "} \\\\ det.\\ ", datap$determiner[this.label], " ", datap$det.date[this.label],  sep="")
   }
  ## If there are attribute data, add these...
  if (!is.na(datap$atts[this.label])) {line <- paste(line, "\\\\ ", datap$atts[this.label], sep="")}
  ## Finish label...
  line <- paste(line, "}", sep="")
  write(line, file.name, append=TRUE)
  }
 line <- "\\end{multicols}
 \\end{document}"
 write(line, file.name, append=TRUE)
}


 ## Now process the file...
 command <- paste("xelatex", file.name)
 print(paste("command:", command))
 system(command, show.output.on.console = FALSE)

 ## Now I need to make a batch CSV file...
 #if (type=="vial") 
  #{
  OTHER_ID_TYPE <- rep("catalog number", l)
  OTHER_ID_NUMBER <- datap$cat_num
  Collection_Cde <- datap$colcode
  Institution_Acronym <- rep("KNWR", l)
  if (type=="vial") {
   Part_Name <- rep("whole organism (vial)", l)
   }
  else {
   Part_Name <- rep("whole organism (pinned)", l)
   }
  BARCODE <- barcode
  PRINT_FG <- rep(0, l)
  if (type=="vial") {
   NEW_CONTAINER_TYPE <- rep("vial", l)
   }
  else {
   NEW_CONTAINER_TYPE <- rep("pin", l)
   }
  to.upload <- as.data.frame(cbind(OTHER_ID_TYPE, Collection_Cde, Institution_Acronym, Part_Name, BARCODE, OTHER_ID_NUMBER, PRINT_FG, NEW_CONTAINER_TYPE))
  upload.file.name <- sub(".csv", "_upload.csv", input.file)
  write.csv(to.upload, upload.file.name, row.names=FALSE)
  #} 
 }






